#ce script contient les fonctions utilisees dans le script6

############################################################################
# fonction depe1_pattern_attendu
#
# cette fonctionrecherche le pattern suivant
# le paragraphe commence par des chiffres suivis 
# d'un point, un blanc et une majuscule
#
# elle retourne un dataframe comprenant deux nouvelles variables 
# deb_cas renseigne s'il s'agit d'un debut de depeche
# num_cas renseigne le numero de depeche temporaire
#
###########################################################################

depe1_pattern_attendu <- function(df_txt_sty_chr_nbp,num_tome){
  pattern_dep <- "^[0-9]+\\.\\s[A-Z]"
  
  df_txt_sty_nbp_chr_dep <- df_txt_sty_chr_nbp %>%
    mutate(
      deb_cas=str_detect(texte, pattern_dep),
     #numero de depeche temporaire si elle est definie par deb_cas
     num_cas = init_dep[num_tome] + cumsum(deb_cas) 
     )
  return(df_txt_sty_nbp_chr_dep)
}



############################################################################
# fonction depe2_visualisation
#
# cette fonction retourne un dataframe permettant de visualiser 
# les numeros de depeche obtenus
# en regard du premier paragraphe de la depeche
#
# cette fonction est utilisee entre deux etapes 
# de mise a jour des variables deb_cas et num_cas 
###########################################################################

depe2_visualisation <- function(df_txt_sty_nbp_chr_dep){
  
  visua <- df_txt_sty_nbp_chr_dep %>%
    mutate( 
      #numero de depeche
      num_dep = num_cas , 
      #nb de paragraphes dans la depeche
      v1 = 1 ,
      nb_parad = ave(v1, num_dep, FUN = sum),
      #numero de paragraphe dans la depeche
      num_parad = ave(v1, num_dep, FUN = seq_along)
    ) %>%
    select(num_dep, num_dep, texte, num_parad) %>%
    filter(num_parad==1)
  return(visua)
}




############################################################################
# fonction depe3_pattern_capita
#
# cette fonctionrecherche le pattern suivant
# le paragraphe contient une proportion élévée de lettres majuscules
#
# elle retourne un dataframe comprenant la mise a jour de deb_cas et num_cas 
# deb_cas renseigne s'il s'agit d'un debut de depeche
# num_cas renseigne le numero de depeche temporaire
#
###########################################################################

depe3_pattern_capita <- function(df_txt_sty_nbp_chr_dep,num_tome){

  df_txt_sty_nbp_chr_dep <- df_txt_sty_nbp_chr_dep %>%
    mutate(
      #nombre de lettres majuscules dans le paragraphe
      capit = sapply(str_match_all(texte,"[A-Z]"),length),
      #nombre de lettres dans le paragraphe
      lettre =sapply(str_match_all(texte,"[A-Za-z]"),length),
      #proportion de lettres majuscules dans le paragraphe
      p = capit/lettre, 
      #casnew
      casnew = (p>=0.63 & lettre>82) |
        (p>=0.64 & lettre>80) | 
        (p>=0.70 & lettre>55) |
        (p>=0.89 & lettre>35) |
        (p>=0.90 & lettre>30) |  
        (p==1 & lettre>29) , 
      ##flag debut de depeche si elle est definie egalement par casnew
      deb_cas = (deb_cas==1)|(casnew==1), 
      #numero de depeche si elle est definie egalement par casnew
      num_cas = init_dep[num_tome] + cumsum(deb_cas) 
      ) %>%
    select(-capit,-lettre,-p,-casnew)
  return(df_txt_sty_nbp_chr_dep)
}




############################################################################
# fonction depe4_pattern_capitb
#
# cette fonctionrecherche le pattern suivant
# le paragraphe contient une proportion élévée de lettres majuscules
# dans les 35 premiers caracteres
#
# elle retourne un dataframe comprenant la mise a jour de deb_cas et num_cas 
# deb_cas renseigne s'il s'agit d'un debut de depeche
# num_cas renseigne le numero de depeche temporaire
###########################################################################

depe4_pattern_capitb <- function(df_txt_sty_nbp_chr_dep,num_tome){

  df_txt_sty_nbp_chr_dep <- df_txt_sty_nbp_chr_dep %>%
    mutate(
    truc=str_sub(texte,1,35),
    long=sapply(str_match_all(texte,"[A-Za-z]"),length),
    #nombre de lettres majuscules dans le paragraphe
    capit = sapply(str_match_all(truc,"[A-Z]"),length),
    #nombre de lettres dans le paragraphe
    lettre =sapply(str_match_all(truc,"[A-Za-z]"),length),
    #proportion de lettres majuscules dans le paragraphe
    p = capit/lettre, 
    #casnew
    casnew=(long>=35 & p>=0.77),
    ##flag debut de depeche si elle est definie en plus par casnew
    deb_cas = (deb_cas==1 | casnew==1), 
    #numero de depeche si elle est definie par en plus par casnew
    num_cas = init_dep[num_tome] + cumsum(deb_cas) 
   ) %>%
  select(-capit,-lettre,-p,-long,-truc,-casnew)
return(df_txt_sty_nbp_chr_dep)
}




############################################################################
# fonction depe5_pattern_cascond
#
# cette fonction recherche le pattern suivant
# le paragraphe precedent n'est pas un debut de depeche 
# le paragraphe precedent celui-ci n'est pas non plus un debut de depeche 
#
# elle retourne un dataframe comprenant la mise a jour de deb_cas et num_cas 
# deb_cas renseigne s'il s'agit d'un debut de depeche
# num_cas renseigne le numero de depeche temporaire
###########################################################################

depe5_pattern_cascond <- function(df_txt_sty_nbp_chr_dep,num_tome){
  
  df_txt_sty_nbp_chr_dep <- df_txt_sty_nbp_chr_dep %>%
    mutate(
    #deb_cas des paragraphes precedent et preprecedent
    pprec = lag(deb_cas,1) ,
    ppprec = lag(deb_cas,2) ,
    #variable cascond le paragraphe n'est pas une suite de debut de depeche
    cascond = (pprec!=1) & (ppprec!=1) ,
    ##flag debut de depeche 
    deb_cas = (deb_cas==1) & (cascond==1)  , 
    #numero de depeche cas
    num_cas = init_dep[num_tome] + cumsum(deb_cas) 
    ) %>%
    select(-pprec,-ppprec,-cascond)
return(df_txt_sty_nbp_chr_dep)
}


############################################################################
# fonction depe6_fn
#
# cette fonction traite manuellement les cas faux négatifs
# non traitables de maniere automatique
#
# elle retourne un dataframe comprenant la mise a jour de deb_cas et num_cas 
# deb_cas renseigne s'il s'agit d'un debut de depeche
# num_cas renseigne le numero de depeche temporaire
###########################################################################

depe6_fn <- function(df_txt_sty_nbp_chr_dep,num_tome){
  df_txt_sty_nbp_chr_dep
  df_txt_sty_nbp_chr_dep <- df_txt_sty_nbp_chr_dep %>%
    mutate(
    cas_fn = 
      (num_tome==8  & num_page==195  & num_parap==1)|
      (num_tome==9  & num_page==71   & num_parap==4)|
      (num_tome==9  & num_page==318  & num_parap==3)|
      (num_tome==10 & num_page==108  & num_parap==7)|
      (num_tome==10 & num_page==120  & num_parap==6)|
      (num_tome==10 & num_page==147  & num_parap==2)|
      (num_tome==10 & num_page==174  & num_parap==6)|
      (num_tome==10 & num_page==234  & num_parap==5)|
      (num_tome==11 & num_page==60   & num_parap==3)|
      (num_tome==11 & num_page==262  & num_parap==3)|
      (num_tome==11 & num_page==365  & num_parap==5)|
      (num_tome==12 & num_page==210  & num_parap==5) ,
    
    ##flag debut de depeche 
    deb_cas = (deb_cas==1) | (cas_fn==1)  , 
    #numero de depeche cas
    num_cas = init_dep[num_tome] + cumsum(deb_cas) 
    ) %>%
    select(-cas_fn)
 
   return(df_txt_sty_nbp_chr_dep)
}

############################################################################
# fonction depe7_fp
#
# cette fonction traite manuellement les cas faux positifs
# non traitables de maniere automatique
#
# elle retourne un dataframe comprenant la mise a jour de deb_cas et num_cas 
# deb_cas renseigne s'il s'agit d'un debut de depeche
# num_cas renseigne le numero de depeche temporaire
###########################################################################

depe7_fp <- function(df_txt_sty_nbp_chr_dep,num_tome){
  
  df_txt_sty_nbp_chr_dep <- df_txt_sty_nbp_chr_dep %>%
    mutate(
      cas_fp =
        (num_tome==8  & num_page==284 & num_parap==1) |
        (num_tome==9  & num_page==43  & num_parap==14) |
        (num_tome==10 & num_page==108 & num_parap>=3 & num_parap<=6) |
        (num_tome==11 & num_page==59  & num_parap>=5 & num_parap<=6) |
        (num_tome==11 & num_page==60  & num_parap>=1 & num_parap<=2) ,

      ##flag debut de depeche 
      deb_cas = (deb_cas==1) & (cas_fp==0)  , 
      #numero de depeche cas
      num_cas = init_dep[num_tome] + cumsum(deb_cas) 
      ) %>%
    select(-cas_fp)
 
   return(df_txt_sty_nbp_chr_dep)
}



############################################################################
# fonction depe8_final
#
# cette fonction retourne un dataframe comprenant trois nouvelle variable
# num_dep    numero de la depeche
# nb_parad   nombre de paragraphes dans la depeche
# num_parad  numero du paragraphe dans la depeche
# de plus, elle supprime les variables num_cas et deb_cas
###########################################################################

depe8_final <- function(df_txt_sty_nbp_chr_dep){
  
  df_txt_sty_nbp_chr_dep <- df_txt_sty_nbp_chr_dep %>%
    mutate(
      #numero de depeche
      num_dep = num_cas , 
      #nb de paragraphes dans la depeche
      V1 = 1 ,
      nb_parad = ave(V1, num_dep, FUN = sum),
      #numero de paragraphe dans la depeche
      num_parad = ave(V1, num_dep, FUN = seq_along)
      ) %>%
    select(-num_cas,-deb_cas,-V1)

    return(df_txt_sty_nbp_chr_dep)
}
