#ce script contient les fonctions utilisees dans le script9

############################################################################
# fonction titr1_prepa
#
# cette fonction crée des flags de reperage 
# flag_nonderparpag    qui indique s'il s'agit du dernier paragraphe de la depeche
# flag_pardep1a5       qui indique s'il s'agit d'un des 5 premiers paragraphes
# flag_paris           qui indique si le paragraphe contient le pattern paris
# flag_cabrec          qui indique si le paragraphe contient le pattern cabinet ou recu
# flag_court           qui indique s'il s'agitd un paragraphe court  
# flag_tailpol         qui indique si la taille de la police du paragraphe
#                      est inferiuyr ou egale à celle du paragraphe precedent
#                      et inférieure à celle du paragraphe suivant
###########################################################################

titr1_prepa <- function(df_txt_sty_nbp_chr_chr_dep_tdm_met) {

  df_txt_sty_nbp_chr_dep_tdm_met_tit1 <- df_txt_sty_nbp_chr_dep_tdm_met %>% 
  mutate(
    
    #taille de police du paragraphe precedent et des deux paragraphes suivants
    prec_taille_pol=lag(taille_pol,n=1),
    post_taille_pol=lead(taille_pol,n=1),
    #longueur du paragraphe en nb de lettres
    lettre =sapply(str_match_all(texte,"[A-Za-z]"),length),
    #pattern cabinet ou recu
    pattern_cab_rec= or( "^\\(Cabinet", "^\\(Reçu", "^\\(Dir", "^\\( Cabinet" ,"^Cabinet", "^\\(Cabinct", "^\\(Pabinet", "^\\(Cabinel", "^\\(Cahinet", "^\\' \\(Cabinet" ),
    #pattern Paris
    pattern_paris=or("^Paris\\, ","^\\[Paris\\,\\]","^\\[Paris\\]\\,","^Paris\\; ","^Vichy\\, ","^\\[ Paris\\, \\] "),
    
    # creation de flag de repérage 
    flag_nonderparpag    = as.numeric(num_parap!=nb_parap),
    flag_pardep1a5       = as.numeric(num_new_parad >1 & num_new_parad<=5),
    flag_paris           = as.numeric(str_detect(texte, pattern_paris)),
    flag_cabrec          = as.numeric(str_detect(texte, pattern_cab_rec)),
    flag_court           = as.numeric(lettre<35),  
    flag_tailpol         = as.numeric(taille_pol<=prec_taille_pol  & taille_pol<post_taille_pol)
  ) 
return(df_txt_sty_nbp_chr_dep_tdm_met_tit1)
}


############################################################################
# fonction titr2_maedf
#
# cette fonction traite les depeches ecrites par le ministre des affaires etrangeres
# elle cree la variable
# flag_der_cas qui indique si le paragraphe est le dernier paragraphe du titre de depeche
###########################################################################


titr2_maedf <- function(df_txt_sty_nbp_chr_dep_tdm_met_tit1) {

  df_txt_sty_nbp_chr_dep_tdm_met_tit2 <- df_txt_sty_nbp_chr_dep_tdm_met_tit1   %>%
  filter(dep_dip_fr==1 & str_detect(titr_exped,"MAEDF")==1)    %>%
  mutate (
    #cas1  pattern paris 
    flag_cas1=as.numeric(flag_nonderparpag ==1 
                       & flag_pardep1a5==1 
                       & flag_court==1 
                       & flag_paris==1),
    
    #cas2  taille de police
    flag_cas2=as.numeric(flag_cas1==0
                      & flag_nonderparpag ==1
                      & flag_pardep1a5==1
                      & flag_court==1
                      & flag_tailpol==1), 

    #cas
    flag_cas=as.numeric(flag_cas1==1|flag_cas2==1) ,
    #nombre de flag_cas
    flag_nb_cas=ave(flag_cas,num_dep,FUN=sum),
    #somme cumulée de flag_cas
    flag_cum_nbcas=ave(flag_cas,num_dep,FUN=cumsum),
    #dernier cas : dernier paragraphe du titre de depeche
    flag_der_cas=as.numeric(flag_cas==1 & (flag_cum_nbcas==flag_nb_cas))
  )
  return(df_txt_sty_nbp_chr_dep_tdm_met_tit2)
    }    


############################################################################
# fonction titr3_maedf_corr
#
# cette fonction traite les depeches ecrites par le ministre des affaires etrangeres
# elle traite manuellement les cas qui n'ont pas pu etre traites de facon automatique 
# par la fonction titr2_maedf
###########################################################################

titr3_maedf_corr <- function(df_txt_sty_nbp_chr_dep_tdm_met_tit2) {
  
  df_txt_sty_nbp_chr_dep_tdm_met_tit3 <- df_txt_sty_nbp_chr_dep_tdm_met_tit2   %>%
    mutate (
      flag_der_cas=ifelse((num_tome==8 & num_dep==1893), 0,flag_der_cas),
      flag_der_cas=ifelse((num_tome==8 & num_dep==1893 & num_new_parad==2),1,flag_der_cas),
      flag_der_cas=ifelse((num_tome==9 & num_dep==2478), 0,flag_der_cas),
      flag_der_cas=ifelse((num_tome==9 & num_dep==2478 & num_new_parad==2),1,flag_der_cas),
      flag_der_cas=ifelse((num_tome==12 & num_dep==3567), 0,flag_der_cas),
      flag_der_cas=ifelse((num_tome==12 & num_dep==3567 & num_new_parad==2),1,flag_der_cas),
      flag_der_cas=ifelse((num_tome==12 & num_dep==3590), 0,flag_der_cas),
      flag_der_cas=ifelse((num_tome==12 & num_dep==3590 & num_new_parad==2),1,flag_der_cas)
    )
  return(df_txt_sty_nbp_chr_dep_tdm_met_tit3)
}    



############################################################################
# fonction titr4_maedf_corps
#
# cette fonction traite les depeches ecrites par le ministre des affaires etrangeres
# elle cree la variable flag_cor_dep qui indique 
# si le paragraphe fait partie du corps de depeche
###########################################################################

titr4_maedf_corps <- function(df_txt_sty_nbp_chr_dep_tdm_met_tit3) {
  
  df_txt_sty_nbp_chr_dep_tdm_met_tit4 <- df_txt_sty_nbp_chr_dep_tdm_met_tit3   %>%
    mutate(
      flag_cor_dep=ave(flag_der_cas,num_dep,FUN=cumsum),
      flag_cor_dep=as.numeric(flag_cor_dep==1 & flag_der_cas==0)
    ) 
  return(df_txt_sty_nbp_chr_dep_tdm_met_tit4)
}







############################################################################
# fonction titr5_amba
#
# cette fonction traite les depeches ecrites par un agent diplomatique
# elle cree la variable
# flag_der_cas qui indique si le paragraphe est le dernier paragraphe du titre de depeche
###########################################################################

titr5_amba <- function(df_txt_sty_nbp_chr_dep_tdm_met_tit1) {
  
  df_txt_sty_nbp_chr_dep_tdm_met_tit5 <- df_txt_sty_nbp_chr_dep_tdm_met_tit1 %>%
  filter(dep_dip_fr==1 & str_detect(titr_dest,"MAEDF")==1)    %>%
    
    mutate(
      #pattern Cabinet ou Reçu
      flag_cas1=as.numeric(flag_nonderparpag ==1 
                        & flag_pardep1a5==1
                        & flag_court==1
                        & flag_cabrec==1),
      
      #taille de police
      flag_cas2=as.numeric (flag_cas1==0
                        & flag_nonderparpag ==1
                        & flag_pardep1a5==1
                        & flag_court==1
                        & flag_tailpol==1),
      
      #deux paragraphes collés
      flag_cas3=as.numeric(flag_cas1==0 & flag_cas2==0 & 
                           lead (as.numeric(
                           flag_pardep1a5==1
                         & flag_court==0
                         & flag_cabrec==1)
                         ,n=1)),
      #correction necessaire pour la derniere observation
      V1=1,
      num_obs=seq_along(V1),
      nb_obs=sum(V1),  
      flag_cas3=ifelse(num_obs==nb_obs,0,flag_cas3),
    
      #cas
      flag_cas=as.numeric(flag_cas1==1|flag_cas2==1|flag_cas3==1), 
    
    #nombre de flag_cas
    flag_nb_cas=ave(flag_cas,num_dep,FUN=sum),
    #somme cumulée de flag_cas
    flag_cum_nbcas=ave(flag_cas,num_dep,FUN=cumsum),
    #dernier cas: dernier paragraphe du titre de depeche
    flag_der_cas=as.numeric(flag_cas==1 & (flag_cum_nbcas==flag_nb_cas))
    )

  return(df_txt_sty_nbp_chr_dep_tdm_met_tit5)
}


############################################################################
# fonction titr6_amba_corr
#
# cette fonction traite les depeches ecrites par un agent diplomatique
# elle traite manuellement les cas qui n'ont pas pu etre traites de facon automatique 
# par la fonction titr5_amba
###########################################################################

titr6_amba_corr <- function(df_txt_sty_nbp_chr_dep_tdm_met_tit5) {
df_txt_sty_nbp_chr_dep_tdm_met_tit6 <- df_txt_sty_nbp_chr_dep_tdm_met_tit5 %>%
  mutate(
    
    # paragraphes collés ex:cabinet attaché à la ligne precedente n=4
    flag_der_cas=ifelse((num_tome==8 & num_dep==1983), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==8 & num_dep==1983 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2822), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2822 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3115), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3115 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3189), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3189 & num_new_parad==2),1,flag_der_cas),

    # pas de paragraphes  "recu" 
    flag_der_cas=ifelse((num_tome==8 & num_dep==2107), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==8 & num_dep==2107 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2872), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2872 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3172), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3172 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3260), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3260 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3343), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3343 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3636), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3636 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3646), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3646 & num_new_parad==2),1,flag_der_cas),

    # minute.archive ambassade de vienne,berlin etc.
    flag_der_cas=ifelse((num_tome==8 & num_dep==1981), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==8 & num_dep==1981 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==9 & num_dep==2288), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==9 & num_dep==2288 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2679), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2679 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2807), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2807 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3021), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3021 & num_new_parad==3),1,flag_der_cas),

    # divers 
    #particuliere
    flag_der_cas=ifelse((num_tome==10 & num_dep==2847), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==10 & num_dep==2847 & num_new_parad==2),1,flag_der_cas),
    #pas de cabinet
    flag_der_cas=ifelse((num_tome==11 & num_dep==3098), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3098 & num_new_parad==2),1,flag_der_cas),
    #particuliere pas de cabinet
    flag_der_cas=ifelse((num_tome==11 & num_dep==3167), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3167 & num_new_parad==2),1,flag_der_cas),
    
    #deux paragraphes collés  
    flag_der_cas=ifelse((num_tome==8 & num_dep==1943), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==8 & num_dep==1943 & num_new_parad==3),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3438), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3438 & num_new_parad==3),1,flag_der_cas),
    
    #autres 
    flag_der_cas=ifelse((num_tome==9 & num_dep==2486), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==9 & num_dep==2486 & num_new_parad==3),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3092), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==11 & num_dep==3092 & num_new_parad==4),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3430), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3430 & num_new_parad==2),1,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3578), 0,flag_der_cas),
    flag_der_cas=ifelse((num_tome==12 & num_dep==3578 & num_new_parad==3),1,flag_der_cas)
  )

return(df_txt_sty_nbp_chr_dep_tdm_met_tit6)
}




############################################################################
# fonction titr7_amba_corps
#
# cette fonction traite les depeches ecrites par un agent diplomatique
# elle cree la variable flag_cor_dep qui indique 
# si le paragraphe fait partie du corps de depeche
###########################################################################

titr7_amba_corps <- function(df_txt_sty_nbp_chr_dep_tdm_met_tit6) {

  df_txt_sty_nbp_chr_dep_tdm_met_tit7 <- df_txt_sty_nbp_chr_dep_tdm_met_tit6   %>%
    mutate(
      flag_cor_dep=ave(flag_der_cas,num_dep,FUN=cumsum),
      flag_cor_dep=as.numeric(flag_cor_dep==1 & flag_der_cas==0)
    ) 
  return(df_txt_sty_nbp_chr_dep_tdm_met_tit7)
}







