###################################################################
# ce script permet de construire le tableau descriptif 
# donant pour chaque  agent diplomatique
# le nombre total de depeches, envoyees, recues, lettres, telegrammes, 
# ainsi que le nombre de depeches par periode d'un mois
#
# 1) creation du dataframe desc_corres3 contenant les informations 
#    qui seront données dans le tableau
#    ce point est réalisé en 3 etapes: 
#    construction de desc_corres1, desc_corres2 puis desc_corres3
#
# 2) creation du tableau de données ce point est effectué en étapes
#    a) creation de desc_corres dataframe permettant 
#       de prendre en compte l'ordre de presentation des diplomates 
#       dans le tableau final
#    b) calcul des nombres de depeches pour chaque sous tableau
#    c) constitution du tableau final
#
#3)  description des depeches confidentielles particulieres et 
#    faisant partie des négociations secretes
###################################################################




load("D:/memoire m2_LNakache/constitution du corpus/environnements/Rdepdipfr.RData")

setwd("D:/memoire m2_LNakache/descriptif/fichiers_textes")



# chargement des packages
library(dplyr)
library(stringr)
library (rebus)
library (readr)
library(ggraph)
library(igraph)
library(stringdist)
library(tidyverse) 
library(dplyr)
library(tm)
library(rebus)
library(tibble)
library(stringr)
library(lubridate)


###########################################################
# 1)  creation du dataframe  desc_corres3
###########################################################


###########################################################
# creation de desc_corres1 
# dataframe des 1566 depeches diplomatiques francaises
###########################################################

pattern_titre=START %R% one_or_more(one_or_more(WRD) %R% SPC) %R% "à"
pattern_lieu="à" %R% one_or_more(SPC %R% one_or_more(WRD)) %R% END


desc_corres1 <- df_depdipfr %>%
  select(-text,-num_tome, -num_page) %>%
  mutate(
    #creation de la variable ade qui renseigne 
    #si la depeche est envoyé à un ou plusieurs agents diplomatiques
    ade=str_detect(dest,or("aux ADE","aux A")),
    # creation de dest_lieu exped_lieu qui renseigne 
    #la ville de destination ou expedition de la depeche
    dest_lieu=str_extract(titr_dest,pattern_lieu),
    exped_lieu=str_extract(titr_exped,pattern_lieu),
    dest_lieu=str_replace(dest_lieu,"à ", ""),
    exped_lieu=str_replace(exped_lieu,"à ", ""),
    #creation de dest_titre et  exped_titre qui renseigne
    #le titre du destinataire ou de l expediteur
    dest_titre=str_extract(titr_dest,pattern_titre),
    exped_titre=str_extract(titr_exped,pattern_titre),
    dest_titre=str_replace(dest_titre,"à", ""),
    exped_titre=str_replace(exped_titre,"à", ""),
    
    exped_lieu=ifelse(titr_exped==" MAEDF","Paris", exped_lieu ),
    dest_lieu= ifelse(titr_dest ==" MAEDF","Paris", dest_lieu ),
    exped_titre=ifelse(titr_exped==" MAEDF","maedf", exped_titre ),
    dest_titre= ifelse(titr_dest ==" MAEDF","maedf", dest_titre )
  ) 



#liste des numéros de depeches envoyées à plusieurs agents diplomatiques
desc_list_plusieurs_ade <- desc_corres1 %>% filter(ade==1) %>%  select(num_dep, circulaire)
#croisement des variables ade et circulaire
table(desc_corres1$circulaire, desc_corres1$ade) 
#il y a 35 depeches aux ADE dont 13 circulaires
write.table(desc_list_plusieurs_ade, "desc_list_plusieurs_ade.csv", sep=";")


#fin desc_corres1###############################################################




##############################################################################
# creation de desc_corres2 
# exclusion des dépeches envoyées à plusieurs agents diplomatiques
# création dela variable periode2 (par mois)
##############################################################################

desc_corres2 <- desc_corres1 %>%
  filter(ade==0) %>%
  mutate(maedf=0,
         maedf=ifelse(titr_exped==" MAEDF",1, maedf ),
         periode=1,
         periode=ifelse(num_dep>=1979 & num_dep<=2067,2,periode),
         periode=ifelse(num_dep>=2068 & num_dep<=2189,3,periode),
         periode=ifelse(num_dep>=2190 & num_dep<=2352,4,periode),
         periode=ifelse(num_dep>=2353 & num_dep<=2509,5,periode),
         periode=ifelse(num_dep>=2510 & num_dep<=2701,6,periode),
         periode=ifelse(num_dep>=2702 & num_dep<=2844,7,periode),
         periode=ifelse(num_dep>=2845 & num_dep<=3007,8,periode),
         periode=ifelse(num_dep>=3009 & num_dep<=3245,9,periode),
         periode=ifelse(num_dep>=3246 & num_dep<=3416,10,periode),
         periode=ifelse(num_dep>=3417 & num_dep<=3538,11,periode),
         periode=ifelse(num_dep>=3542 & num_dep<=3595,12,periode),
         periode=ifelse(num_dep>=3599 & num_dep<=3642,13,periode),
         periode=ifelse(num_dep>= 3643,14,periode),
         periode2=floor((periode+1)/2)            
         )
#fin desc_corres2###############################################################



##########################################################################
# creation de desc_corres3
# 1) les caracteristiques de la depeche sont 
#    le nom du diplomate qu'il soit expediteur ou destinataire
#    la ville ou il est en poste
#   son titre (fonction) :ambassadeur, ministre etc.
# 2) modification pour le baron de Bouquenay  
#    pour lequel deux titres ont été trouvés dans les notices
##########################################################################

desc_corres3a <- desc_corres2 %>%
  filter(type=="adep") %>%
  mutate(nom=str_trim(exped),
         lieu=str_trim(exped_lieu),
         titre=str_trim(exped_titre)) %>%
  select(num_dep,type,nom,titre,lieu,teleg,periode,periode2,nb_mots,particuliere,negosec,confidentielle,date,date_c)

desc_corres3b <- desc_corres2 %>%
  filter(type=="maedf") %>%
  mutate(nom=str_trim(dest),
         lieu=str_trim(dest_lieu),
         titre=str_trim(dest_titre)) %>%
  select(num_dep,type,nom,titre,lieu,teleg,periode,periode2,nb_mots,particuliere,negosec,confidentielle,date,date_c)

desc_corres3 <-rbind(desc_corres3a,desc_corres3b) %>%
  mutate(titre=ifelse(nom=="Le Baron de Bourqueney","Chargé d'Affaires",titre)) 

rm(desc_corres3a,desc_corres3b )
#fin creation desc_corres3#########################################################################





###########################################################
# 2)  creation du tableau de donnees
###########################################################


#l objectif est de constituer un tableau decrivant les caracteristiques des depeches
#le choix a été fait de les presenter par diplomate et de classer les diplomates par pays
#en presentant d' abord les protagonistes (prusse et autriche),
#puis l italie et la confederation germanique
#et enfin l'europe et l empire ottoman, 






#####################################################
# 2)a creation de desc_corres 
########################################################

# lecture de ade.txt
# le fichier "les noms des ade.txt" a été créé à la main  
# il contient 5 variables : pays ville nom titre et num_nom 
# le numero du diplomate qui donne l'ordre d'apparition 
# du diplomatedans le tableau descriptif
ade=read.csv("les noms des ade.txt",  sep =";", stringsAsFactors = F)

#creation de desc_corres par merge de desc_corres3 et ade
desc_corres <- merge(desc_corres3,ade, by="nom") 
desc_corres$date=as.Date.numeric(desc_corres$date,origin = "1970-01-01")


#################################################################################
#  2)b Création des sous tableaux resultats 

#par agent diplomatique nombre total de depeces 
tableau1<- as.data.frame(table(desc_corres$nom)) %>% rename(nom=Var1,total=Freq)
tableau1 <- merge(ade,tableau1,by="nom" ) %>% arrange(num_nom) %>%
  select(pays,ville,nom,num_nom,titre,total)

#par agent diplomatique nombre de depeches Envoyées, Recues
options( "digits"=2)
tableau2<- as.data.frame(table(desc_corres$nom,desc_corres$type)) %>% rename(nom=Var1)
tableau2a <-tableau2 %>% filter(Var2=="adep") %>% rename(envoyées=Freq) %>% select(-Var2)
tableau2b <-tableau2 %>% filter(Var2=="maedf") %>% rename(reçues=Freq) %>% select(-Var2)
tableau2 <- merge(tableau2a,tableau2b,by="nom") %>%
  mutate(prop2=reçues/(envoyées+reçues),tot2=envoyées+reçues) 
tableau2 <- merge(tableau2,ade,by="nom") %>% arrange(num_nom) %>%
  select(pays,ville,nom,titre,envoyées,reçues,tot2,prop2) 


#par agent diplomatique lettres telegrammes
tableau3<- as.data.frame(table(desc_corres$nom,desc_corres$teleg)) %>% rename(nom=Var1)
tableau3a <-tableau3 %>% filter(Var2==0) %>% rename(lettres=Freq) %>% select(-Var2)
tableau3b <-tableau3 %>% filter(Var2==1) %>% rename(télégrammes=Freq) %>% select(-Var2)
tableau3 <- merge(tableau3a,tableau3b,by="nom") %>%
  mutate(tot3=lettres+télégrammes,
         prop3=télégrammes/tot3)
tableau3 <- merge(tableau3,ade,by="nom") %>% arrange(num_nom)%>%
  select(pays,ville,nom,titre,lettres,télégrammes,tot3,prop3)


#par agent diplomatique periode d'un mois
options( "digits"=2)
tableau4<- as.data.frame(table(desc_corres$nom,desc_corres$periode2)) %>% rename(nom=Var1)
tableau4a <-tableau4 %>% filter(Var2==1) %>% rename(p1=Freq) %>% select(-Var2)
tableau4b <-tableau4 %>% filter(Var2==2) %>% rename(p2=Freq) %>% select(-Var2)
tableau4c <-tableau4 %>% filter(Var2==3) %>% rename(p3=Freq) %>% select(-Var2)
tableau4d <-tableau4 %>% filter(Var2==4) %>% rename(p4=Freq) %>% select(-Var2)
tableau4e <-tableau4 %>% filter(Var2==5) %>% rename(p5=Freq) %>% select(-Var2)
tableau4f <-tableau4 %>% filter(Var2==6) %>% rename(p6=Freq) %>% select(-Var2)
tableau4g <-tableau4 %>% filter(Var2==7) %>% rename(p7=Freq) %>% select(-Var2)

tableau4 <- merge(tableau4a,tableau4b,by="nom",all.x=TRUE,all.y=TRUE)
tableau4 <- merge(tableau4, tableau4c,by="nom",all.x=TRUE,all.y=TRUE)
tableau4 <- merge(tableau4 ,tableau4d,by="nom",all.x=TRUE,all.y=TRUE)
tableau4 <- merge(tableau4, tableau4e,by="nom",all.x=TRUE,all.y=TRUE)
tableau4 <- merge(tableau4, tableau4f,by="nom",all.x=TRUE,all.y=TRUE)
tableau4 <- merge(tableau4 ,tableau4g,by="nom",all.x=TRUE,all.y=TRUE)
tableau4 <- merge(tableau4,ade,by="nom",all.x=TRUE,all.y=TRUE) %>% arrange(num_nom) %>%
  select(pays,ville,nom,titre,p1,p2,p3,p4,p5,p6,p7) %>%
  mutate(tot4=p1+p2+p3+p4+p5+p6+p7,
         pp1=p1/tot4,
         pp2=p2/tot4,
         pp3=p3/tot4,
         pp4=p4/tot4,
         pp5=p5/tot4,
         pp6=p6/tot4,
         pp7=p7/tot4) 


#fin constitution des sous-tableaux 


#nettoyage de l'environnement
rm(tableau2a,tableau2b)
rm(tableau3a,tableau3b)
rm(tableau4a,tableau4b,tableau4c,tableau4d,tableau4e,tableau4f,tableau4g)



##########################################################
# 2)c constitution du tableau final 
#######################################################

# merge des sous tableaux crees a l etape precedente
tableau <-merge(tableau1,tableau2%>%select(nom,envoyées,reçues),by="nom")
tableau <-merge(tableau,tableau3%>%select(nom,lettres,télégrammes),by="nom")
tableau <-merge(tableau,tableau4%>%select(nom,p1,p2,p3,p4,p5,p6,p7),by="nom") %>%
  arrange(num_nom) %>%
  select(pays,ville,nom,titre,total,envoyées,reçues,lettres,télégrammes,p1,p2,p3,p4,p5,p6,p7)

# nettoyage de l'environnement
rm(tableau1,tableau2,tableau3,tableau4)

#Calcul de la ligne Total
lignetot <-c(
  c("Total",NA,NA,NA),
  colSums(tableau%>% select(total,envoyées,reçues,lettres,télégrammes,p1,p2,p3,p4,p5,p6,p7))
  )

#ajout de la ligne total
tableau <-rbind(tableau,lignetot)


#presentation des sous tableaux par groupe de pays 
tableau_prusseautriche <- rbind( c("Les deux principaux belligerants",NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA)
                               , tableau %>% slice(1:6) ) 
tableau_italie         <- rbind( c("Italie",NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA)
                                 , tableau %>% slice(7:11) ) 
tableau_confger        <- rbind( c("Confédération germanique",NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA)
                                 , tableau %>% slice(12:30) ) 
tableau_europe         <- rbind( c("Europe",NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA)
                                 , tableau %>% slice(31:42) ) 
tableau_empotto        <- rbind( c("Empire Ottoman",NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA)
                                 , tableau %>% slice(43:47) ) 
tableau_ligntot        <- rbind( c(NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA,NA)
                                , tableau %>% slice(48) ) 

#tableau final
tableau_final <-rbind(tableau_prusseautriche,tableau_italie,tableau_confger,tableau_europe,tableau_empotto,tableau_ligntot)

# nettoyage de l'environnement
rm(lignetot)
rm(tableau_prusseautriche,tableau_italie,tableau_confger,tableau_europe,tableau_empotto,tableau_ligntot)



write.table(tableau_final, "tableau_final.csv", sep=";")
#fin constitution du tableau final ##########################################################################################



####################################################
# 3)  description des depeches confidentielles particulieres  
#     et faisant partie des négociations secretes
####################################################

#confidentielles
tableau_lettresconfidentielles <- desc_corres %>%
  filter(confidentielle==1) %>%
  arrange(num_nom,num_dep) %>%
  select(num_nom,nom,num_dep,type,date,teleg)

#particulières
tableau_lettresparticulieres <- desc_corres %>%
  filter(particuliere==1) %>%
  arrange(num_nom,num_dep) %>%
  select(num_nom,nom,num_dep,type,date,teleg)
table(tableau_lettresparticulieres$nom, tableau_lettresparticulieres$type)


#negociations secretes
tableau_negosiationssecretes<- desc_corres %>%
  filter(negosec==1) %>%
  arrange(num_nom,num_dep) %>%
  select(num_nom,nom,num_dep,type,date,teleg)
table(tableau_negosiationssecretes$type, tableau_negosiationssecretes$teleg)



