###ce script permet de réaliser l'analyse stylométrique du style diplomatique pendant sadowa ###



load("D:/memoire m2_LNakache/constitution du corpus/environnements/Rdepdipfr.RData")
setwd("D:/memoire m2_LNakache/stylometrie/mots")




#librairies
library(tidyverse) 
library(dplyr)
library(tm)
library(rebus)
library(cluster)
library("dendextend")



######selection des lettres pour étude #############################
# on ote les dépeches avec crochet et les télégrammes
# et on ne conserve que les 4 agents diplomatiques
# Benedetti, Gramont, La Tour d'Auvergne et Talleyrand
df_stylo1 <- df_depdipfr %>%
  mutate(
    crochet=str_detect(text, or("\\[", "\\]")),
    #somme par num de depeche de la variable crochet
    dep_crochet=ave(crochet,num_dep,FUN=sum) 
  )%>%
  #selection des lettres sans crochets pour 4 agents diplomatiques
  filter(teleg==0 & dep_crochet==0
        & (exped == "Benedetti"| exped == "Le Duc de Gramont"
         | exped == "Le Prince de La Tour d Auvergne"
         | exped == "Le Baron de Talleyrand") ) %>%
   #on remplace les apostrophes par des espaces
    mutate(
    text=str_replace_all(text,"\\'"," ")
  )



###################################################################
# construction du df df_stylo2 nécessaire à la classification
# titage au sort de 22 depeches par expediteur

#declaration du set seed
set.seed(1789)

#tirage au sort pour n'avoir que 22 depeches par diplomate
df_stylo2 <- df_stylo1 %>%
  mutate(random=runif(210)) %>%
  group_by(exped) %>%
  top_n(22,random) %>%
  ungroup() %>%
  select(-random) %>%
  mutate( doc_id = num_dep) %>%
  select(doc_id,text,exped)
####fin de la construction du df nécessaire à la classification



###### mise en corpus #############################################
#source import
df_source = DataframeSource(df_stylo2)
# à cette étape on intègre les métadonnées plus les textes dans un objet corpus:
df_corpus = VCorpus(df_source,readerControl = list( language = "fr"))
#on enlève la ponctuation
df_corpus = tm_map(df_corpus, removePunctuation)


#################################################################
# 1) creation de stopcontenu

#on convertit en TDM et argument control pour garder les majuscules
depeche_tdm = TermDocumentMatrix(df_corpus, control = list(tolower = F))
dim(depeche_tdm)   #  6485   88
# mise sous forme de matrice
matrix_tdm = as.matrix(depeche_tdm)
#puis de dataframe
df_tdm = as.data.frame(matrix_tdm)
#creation de term_freq somme des lignes, qu'on ordonne ensuite
df_tdm$term_freq = rowSums(df_tdm)
df_tdm_ordo = df_tdm[order(df_tdm$term_freq, decreasing = T), ]
#suppression term_freq car non nécessaire
df_tdm_ordo = df_tdm_ordo %>% select(-term_freq)

#selection des 200 premiers mots pour pouvoir nettoyer les mots de contenu
df_tdm_sub = df_tdm_ordo[1:200,] 

###################################################################
#creation d un fichier csv contenant les 200 premiers mots 
# et une colonne style qui sera retravaillée manuellement
mots = as.data.frame(rownames(df_tdm_sub))
mots = mots %>% mutate(style = 1)
write.table(mots, "mots.csv", sep=",")


#####################################################################
# lecture de fichier mots_2, fichier csv 
# où l'on a annoté manuellement le style des mots, 
# en 0 si mots du contenu, 1 si mots du style, 2 si ambigu
mots_2=read.csv("mots_2.csv",  sep =",", stringsAsFactors = F)


#####################################################################
#creation de stopcontenu vecteur les mots du contenus ou les mots ambigus
bool=mots_2$style!=1
stopcontenu=mots_2$mots[bool]



##fin creation de stopcontenu
#####################################################################





#####################################################################
#  2) on repasse une partie du script en retirant les mots du contenu ; 
#  TDM , termfreq, puis subset de la matrice 
#  pour obtenir df_tdm_sub


#on enlève les mots de stopcontenu
df_corpus = tm_map(df_corpus, removeWords, stopcontenu)

#TDM
depeche_tdm = TermDocumentMatrix(df_corpus, control = list(tolower = F))
dim(depeche_tdm)   #  6416   88
# mise sous forme de matrice
matrix_tdm = as.matrix(depeche_tdm)
#puis en df
df_tdm = as.data.frame(matrix_tdm)
#creation de term_freq somme des lignes, qu'on ordonne ensuite
df_tdm$term_freq = rowSums(df_tdm)
df_tdm_ordo = df_tdm[order(df_tdm$term_freq, decreasing = T), ]
#suppression term_freq car non nécessaire
df_tdm_ordo = df_tdm_ordo %>% select(-term_freq)


# après avoir regardé les 100 premiers, 
# on trouve encore deux termes se rapportant au contenu on les supprime manuellement
df_tdm_ordo=df_tdm_ordo[!rownames(df_tdm_ordo) %in% c("négociations","dispositions"),]
#on ne garde que 100 premières lignes, celles des mots outils
df_tdm_sub = df_tdm_ordo[1:100,] 
############################################################################



############################################################################
#3) ANALYSE CAH   


#on passe en matrice
mat_tdm_sub  <- as.matrix(df_tdm_sub)

#on travaille en pourcentage
for(i in 1:ncol(mat_tdm_sub)){
  mat_tdm_sub[,i] = mat_tdm_sub[,i]/sum(mat_tdm_sub[,i])
}

#on effectue une Z-transformation
for(i in 1:nrow(mat_tdm_sub)){
  mat_tdm_sub[i,] = ( mat_tdm_sub[i,] - mean(mat_tdm_sub[i,]) )  / sd(mat_tdm_sub[i,])
}

#une normalisation euclidienne
for(i in 1:ncol(mat_tdm_sub)){
  mat_tdm_sub[,i] = mat_tdm_sub[,i] / sqrt(sum(mat_tdm_sub[,i]^2)) 
}
dtm = as.data.frame(t(mat_tdm_sub))


dim(dtm)# 88  100


#on ajoute les noms d'expéditeurs au dataframe dtm 
dtm_meta = dtm %>% 
  mutate(
    exped = df_stylo2$exped
  )


# cah et dendogramme
mycah= agnes(dtm_meta[,1:100], metric = "euclid", method="ward")
mydend = as.dendrogram(mycah)

#parametrage des couleurs et plot du dendogramme
color=vector(length = nrow(dtm_meta))
color[dtm_meta$exped=="Benedetti"]="red"
color[dtm_meta$exped=="Le Baron de Talleyrand"]="green"
color[dtm_meta$exped=="Le Duc de Gramont"]="blue"
color[dtm_meta$exped=="Le Prince de La Tour d Auvergne"]="purple"

labels_colors(mydend) = color[order.dendrogram(mydend)]
plot(mydend, main="Dendrogramme CAH", 
     sub="88 dépêches des tomes 8 à 12 - quatre ambassadeurs")


#création du vecteur cluster qui associe un cluster à chaque dépêche 
#à l'aide de cutree(), h= hauteur du dendrogramme
cluster= as.vector(cutree(mydend,k=6))


#creation du tableau croisant classes et expediteurs
df_clust = as.data.frame.matrix(table( dtm_meta$exped,cluster))
row.names(df_clust) <- c("Benedetti", "Le Baron de Talleyrand","Le Duc de Gramont","Le Prince de La Tour d Auvergne")
#test de chi 2 croisement classes expediteurs
xsq <-chisq.test(df_clust, correct=TRUE)
xsq



#visualisation des six classes attendues sur le dendrogramme
colorc=vector(length = nrow(dtm_meta))
colorc[cluster==1]="red"
colorc[cluster==2]="green"
colorc[cluster==3]="blue"
colorc[cluster==4]="purple"
colorc[cluster==5]="orange"
colorc[cluster==6]="black"

labels_colors(mydend) = colorc[order.dendrogram(mydend)]
plot(mydend, main = "Répartion des six classes restenues")


